package com.smppw.modaq.application.components.report.parser.pdf;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.smppw.modaq.application.components.ReportParseUtils;
import com.smppw.modaq.application.components.report.parser.ReportParserConstant;
import com.smppw.modaq.domain.dto.report.*;
import com.smppw.modaq.domain.mapper.EmailFieldMappingMapper;
import org.springframework.stereotype.Component;
import technology.tabula.Table;

import java.util.List;
import java.util.Map;
import java.util.Objects;

@Component(ReportParserConstant.PARSER_PDF_LETTER)
public class PDLetterReportParser extends AbstractPDReportParser<LetterReportData> {
    private final Map<String, Object> allInfoMap = MapUtil.newHashMap(32);

    public PDLetterReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        super(fieldMappingMapper);
    }

    @Override
    protected void initTableInfo(List<Table> tables) {
        // 每次重新清空map数据
        this.allInfoMap.clear();
        if (CollUtil.isEmpty(tables)) {
            if (StrUtil.isNotBlank(this.aiParserContent)) {
                JSONObject jsonObject = JSONUtil.parseObj(this.aiParserContent);
                this.allInfoMap.putAll(flattenMap(jsonObject, ListUtil.list(false)));
            }
            return;
        }
        Table table = tables.get(0);
        if (table == null) {
            return;
        }
        int rowCount = table.getRowCount();
        int colCount = table.getColCount();
        for (int i = 0; i < rowCount; i++) {
            int t = colCount / 2;
            for (int j = 0; j < t; j++) {
                String key = table.getCell(i, j * 2).getText().replaceAll("[a-zA-Z]", "");
                key = ReportParseUtils.cleaningValue(key);
                if (StrUtil.isBlank(key)) {
                    continue;
                }
                this.allInfoMap.put(key, ReportParseUtils.cleaningValue(table.getCell(i, j * 2 + 1).getText()));
            }
        }
    }

    @Override
    protected ReportFundInfoDTO buildFundInfo(ReportParserParams params) {
        ReportFundInfoDTO fundInfo = this.buildDto(params.getFileId(), ReportFundInfoDTO.class, this.allInfoMap);
        if (CollUtil.isNotEmpty(this.textList) && fundInfo.getFundName() == null) {
            fundInfo.setFundName(this.textList.get(0));
        }
        return fundInfo;
    }

    @Override
    protected LetterReportData parseExtInfoAndSetData(ReportBaseInfoDTO reportInfo, ReportFundInfoDTO fundInfo) {
        Integer fileId = reportInfo.getFileId();
        // 投资者信息
        ReportInvestorInfoDTO investorInfo = this.buildDto(fileId, ReportInvestorInfoDTO.class, this.allInfoMap);
        // 交易流水
        ReportFundTransactionDTO fundTransaction = this.buildDto(fileId, ReportFundTransactionDTO.class, this.allInfoMap);
        // 构建结果数据
        LetterReportData reportData = new LetterReportData(reportInfo, fundInfo);
        reportData.setInvestorInfo(investorInfo);
        reportData.setFundTransaction(fundTransaction);
        reportData.setAiParse(Objects.equals(true, this.aiParse));
        reportData.setAiFileId(this.aiFileId);
        return reportData;
    }

    private static Map<String, Object> flattenMap(Map<String, Object> data, List<String> keys) {
        Map<String, Object> result = MapUtil.newHashMap(16);
        for (Map.Entry<String, Object> entry : data.entrySet()) {
            List<String> currKeys = ListUtil.toList(keys);
            currKeys.add(entry.getKey());
            if (entry.getValue() instanceof Map<?, ?>) {
                result.putAll(flattenMap((Map<String, Object>) entry.getValue(), currKeys));
            } else {
                result.put(entry.getKey(), entry.getValue());
            }
        }
        return result;
    }
}
